@@ -1,5 +1,3 @@
-#![doc = include_str!("../README.md")]
-
 use std::fmt::{self, Error, Formatter};
 
 use borsh::{BorshDeserialize, BorshSerialize};
@@ -7,7 +5,7 @@ use near_account_id::AccountId;
 use near_rpc_error_macro::RpcError;
 use serde::{Deserialize, Serialize};
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, BorshDeserialize, BorshSerialize)]
 pub enum VMError {
     FunctionCallError(FunctionCallError),
     /// Serialized external error from External trait implementation.
@@ -19,7 +17,8 @@ pub enum VMError {
     CacheError(CacheError),
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+// TODO(4217): remove borsh serialization derives, once fix compilation caching.
+#[derive(Debug, Clone, PartialEq, Eq, BorshDeserialize, BorshSerialize)]
 pub enum FunctionCallError {
     /// Wasm compilation error
     CompilationError(CompilationError),
@@ -66,7 +65,9 @@ pub enum FunctionCallErrorSer {
     ExecutionError(String),
 }
 
-#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+#[derive(
+    Debug, Clone, PartialEq, Eq, BorshDeserialize, BorshSerialize, Deserialize, Serialize, RpcError,
+)]
 pub enum CacheError {
     ReadError,
     WriteError,
@@ -143,9 +144,6 @@ pub enum PrepareError {
     Instantiate,
     /// Error creating memory.
     Memory,
-    /// Contract contains too many functions.
-    #[cfg(feature = "protocol_feature_limit_contract_functions_number")]
-    TooManyFunctions,
 }
 
 #[derive(
@@ -293,9 +291,7 @@ impl fmt::Display for PrepareError {
             GasInstrumentation => write!(f, "Gas instrumentation failed."),
             StackHeightInstrumentation => write!(f, "Stack instrumentation failed."),
             Instantiate => write!(f, "Error happened during instantiation."),
-            Memory => write!(f, "Error creating memory."),
-            #[cfg(feature = "protocol_feature_limit_contract_functions_number")]
-            TooManyFunctions => write!(f, "Too many functions in contract."),
+            Memory => write!(f, "Error creating memory"),
         }
     }
 }
